package com.supervision.police.dto.caseScore;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.supervision.constant.JudgeResultEnum;
import com.supervision.constant.ScoreEnum;
import com.supervision.police.domain.*;
import com.supervision.police.mapper.*;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.springframework.ai.ollama.OllamaChatClient;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 案件得分详情
 */
@Data
public class CaseScoreDetailBuilder {

    @Schema(description = "案件id")
    private String caseId;

    @Schema(description = "案件名称")
    private String caseName;

    @Schema(description = "评估时间")
    private LocalDateTime evaluationTime;


    @Schema(description = "当事人列表")
    private List<LawParty> lawPartyList;

    @Schema(description = "指标结果")
    private List<IndexResult> indexResultList = new ArrayList<>();

    /**
     * Ollama 客户端
     */
    private OllamaChatClient chatClient;

    private ModelCaseMapper modelCaseMapper;
    private CasePersonMapper casePersonMapper;
    private ModelAtomicIndexMapper modelAtomicIndexMapper;
    private ModelIndexResultMapper modelIndexResultMapper;
    private ModelIndexMapper modelIndexMapper;
    private ModelAtomicResultMapper modelAtomicResultMapper;

    public CaseScoreDetailBuilder() {
    }

    public CaseScoreDetailBuilder setOllamaChatClient(OllamaChatClient chatClient) {
        this.chatClient = chatClient;
        return this;
    }

    public CaseScoreDetailBuilder setMapper(ModelCaseMapper modelCaseMapper, CasePersonMapper casePersonMapper,
                                            ModelAtomicIndexMapper modelAtomicIndexMapper,ModelIndexResultMapper modelIndexResultMapper,
                                            ModelIndexMapper modelIndexMapper,ModelAtomicResultMapper modelAtomicResultMapper) {
        this.modelCaseMapper = modelCaseMapper;
        this.casePersonMapper = casePersonMapper;
        this.modelAtomicIndexMapper = modelAtomicIndexMapper;
        this.modelIndexResultMapper = modelIndexResultMapper;
        this.modelIndexMapper = modelIndexMapper;
        this.modelAtomicResultMapper = modelAtomicResultMapper;
        return this;
    }


    /**
     * 共性指标分数
     * @return
     */
    public Integer getCommonScore() {

        return sumIndexGroupScore("1");
    }

    /**
     * 入罪指标分数
     * @return
     */
    public Integer getCrimeScore() {

        return sumIndexGroupScore("2");
    }

    /**
     * 出罪指标分数
     * @return
     */
    public Integer getCrimeOutScore() {
        return sumIndexGroupScore("3");
    }

    /**
     * 获取行为人
     */
    public LawParty getLawActor() {
        if (CollUtil.isEmpty(this.lawPartyList)){
            return null;
        }
        return this.lawPartyList.stream()
                .filter(lawParty -> StrUtil.equals("1",lawParty.getUserType())).findFirst().orElse(null);
    }

    public void setIndexResultList(List<ModelIndex> indexList, List<ModelIndexResult> indexResultList,
                                   List<ModelAtomicIndex> atomicIndexList, List<ModelAtomicResult> atomicResultList) {

        if (CollUtil.isEmpty(indexList) || CollUtil.isEmpty(indexResultList)){
            this.indexResultList = new ArrayList<>();
            return;
        }

        Map<String, ModelIndexResult> modelIndexResultMap = indexResultList.stream().collect(
                Collectors.toMap(ModelIndexResult::getIndexId, modelIndexResult -> modelIndexResult));

       this.indexResultList = indexList.stream().map(indexInfo ->
               new IndexResult(indexInfo,modelIndexResultMap.get(indexInfo.getId()),atomicIndexList,atomicResultList)
       ).toList();


    }

    /**
     * 设置行为人
     * @param casePersonList
     */
    public void setLawActor(List<CasePerson> casePersonList) {
        if (CollUtil.isEmpty(casePersonList)){
            this.lawPartyList = new ArrayList<>();
            return;
        }
        this.lawPartyList = casePersonList.stream()
                .map(casePerson -> {
                    LawParty lawParty = new LawParty();
                    lawParty.setUserId(casePerson.getId());
                    lawParty.setUserName(casePerson.getName());
                    lawParty.setUserType(casePerson.getRoleCode());
                    return lawParty;
                }).toList();
    }


    /**
     * 构建CaseScoreDetailDTO
     */
    public CaseScoreDetailDTO build(){

        CaseScoreDetailDTO caseScoreDetailDTO = new CaseScoreDetailDTO();
        caseScoreDetailDTO.setCaseName(this.caseName);
        caseScoreDetailDTO.setEvaluationTime(this.evaluationTime);
        caseScoreDetailDTO.setCommonScore(this.getCommonScore());
        Integer crimeScore = this.getCrimeScore();
        Integer crimeOutScore = this.getCrimeOutScore();
        caseScoreDetailDTO.setSpecificCrime(crimeScore < crimeOutScore ? "出罪证据指标" : "入罪证据指标");
        caseScoreDetailDTO.setSpecificCrimeScore(crimeScore < crimeOutScore? crimeOutScore:crimeScore);

        int sumScore = caseScoreDetailDTO.getCommonScore() + caseScoreDetailDTO.getSpecificCrimeScore();
        caseScoreDetailDTO.setTotalScore(sumScore);
        ScoreEnum scoreEnum = ScoreEnum.getScoreEnum(sumScore);
        if (scoreEnum != null){
            caseScoreDetailDTO.setScoreDesc(scoreEnum.getDesc());
            caseScoreDetailDTO.setJudicialCharacterization(scoreEnum.getName());
        }

        caseScoreDetailDTO.setCommonIndexCount(this.countIndexGroup("1",res->true).intValue());
        caseScoreDetailDTO.setCommonIndexHitCount(this.countIndexGroup("1",res->res.getActualScore() > 0).intValue());

        if (crimeScore < crimeOutScore){
            caseScoreDetailDTO.setCrimeOrOut(1);
            caseScoreDetailDTO.setSpecificCrimeIndexCount(this.countIndexGroup("3", res->true).intValue());
            caseScoreDetailDTO.setSpecificCrimeIndexHitCount(this.countIndexGroup("3", res->res.getActualScore() > 0).intValue());
        }else {
            caseScoreDetailDTO.setCrimeOrOut(0);
            caseScoreDetailDTO.setSpecificCrimeIndexCount(this.countIndexGroup("2", res->true).intValue());
            caseScoreDetailDTO.setSpecificCrimeIndexHitCount(this.countIndexGroup("2", res->res.getActualScore() > 0).intValue());
        }

        // 构建指标描述
        // 命中的公共指标数据
        List<IndexResult> commonIndexResultList = this.listCommonIndexResult();
        caseScoreDetailDTO.setCommonIndexDescList(
                commonIndexResultList.stream().map(indexResult -> indexResult.buildDesc(this.chatClient)).collect(Collectors.toList()));
        // 命中的入罪或者出罪指标数据
        List<IndexResult> specialCirimeIndexResultList = Integer.valueOf(0).equals(caseScoreDetailDTO.getCrimeOrOut()) ?
                this.listCrimeIndexResult():this.listOutCrimeIndexResult();
        caseScoreDetailDTO.setSpecificCrimeIndexDescList(
                specialCirimeIndexResultList.stream().map(indexResult -> indexResult.buildDesc(this.chatClient)).collect(Collectors.toList())
        );

        caseScoreDetailDTO.setMissAtomicIndexCount(
                this.indexResultList.stream().filter(indexResult -> indexResult.getActualScore() == 0)
                        .flatMap(indexResult -> indexResult.getAtomicResults().stream()).filter(atomicResult -> {
                            JudgeResultEnum instance = JudgeResultEnum.getInstance(atomicResult.getJudgeResult());
                            return instance == null || !instance.flagIsTrue();
                        }).count()
        );
        return caseScoreDetailDTO;
    }

    private Integer sumIndexGroupScore(String indexGroup) {
        if (CollUtil.isEmpty(this.indexResultList)){
            return 0;
        }

        return this.indexResultList.stream()
                .filter(indexResult -> StrUtil.equals(indexGroup,indexResult.getIndexGroup()))
                .mapToInt(IndexResult::getActualScore).sum();

    }

    private Long countIndexGroup(String indexGroup, Function<IndexResult, Boolean> function) {
        if (CollUtil.isEmpty(this.indexResultList)){
            return 0L;
        }

        return this.indexResultList.stream()
                .filter(indexResult -> StrUtil.equals(indexGroup,indexResult.getIndexGroup())
                        && function.apply(indexResult)).count();

    }


    /**
     * 获取共性指标
     * @return
     */
    public List<IndexResult> listCommonIndexResult() {
        return listIndexGroup("1",res->res.getActualScore() > 0);
    }

    /**
     * 获取入罪指标
     * @return
     */
    public List<IndexResult> listCrimeIndexResult() {
        return listIndexGroup("2",res->res.getActualScore() > 0);
    }

    /**
     * 获取出罪指标
     * @return
     */
    public List<IndexResult> listOutCrimeIndexResult() {
        return listIndexGroup("3",res->res.getActualScore() > 0);
    }

    private List<IndexResult> listIndexGroup(String indexGroup, Function<IndexResult, Boolean> function) {
        if (CollUtil.isEmpty(this.indexResultList)){
            return new ArrayList<>();
        }

        return this.indexResultList.stream()
                .filter(indexResult -> StrUtil.equals(indexGroup,indexResult.getIndexGroup())
                        && function.apply(indexResult)).collect(Collectors.toList());

    }

    public CaseScoreDetailBuilder loadCaseScoreDetail(String caseId) {

        // 1 设置案件基本信息
        ModelCase modelCase = modelCaseMapper.selectById(caseId);
        this.setCaseId(modelCase.getId());
        this.setCaseName(modelCase.getCaseName());
        this.setEvaluationTime(modelCase.getCaseAnalysisSuccessTime());
        // 2 设置当事人人员信息
        List<CasePerson> casePersonList = casePersonMapper.selectList(new LambdaQueryWrapper<CasePerson>().eq(CasePerson::getCaseId, caseId));
        this.setLawActor(casePersonList);

        // 3 设置指标结果信息

        // 3.1 获取指标信息
        List<ModelIndex> modelIndices = modelIndexMapper.selectList(new LambdaQueryWrapper<ModelIndex>().eq(ModelIndex::getCaseType, modelCase.getCaseType()).eq(ModelIndex::getDataStatus, "1"));

        // 3.2 获取指标结果信息 后面需要根据行人人id进行过滤
        List<ModelIndexResult> modelIndexResults = modelIndexResultMapper.selectList(
                new LambdaQueryWrapper<ModelIndexResult>().eq(ModelIndexResult::getCaseId, caseId));

        // 3.3 获取原子指标信息
        List<ModelAtomicIndex> modelAtomicIndices = modelAtomicIndexMapper.selectList(
                new LambdaQueryWrapper<ModelAtomicIndex>().eq(ModelAtomicIndex::getCaseType, modelCase.getCaseType())
                        .eq(ModelAtomicIndex::getDataStatus, "1"));

        // 3.4 获取原子指标结果
        List<ModelAtomicResult> modelAtomicResults = modelAtomicResultMapper.selectList(
                new LambdaQueryWrapper<ModelAtomicResult>().eq(ModelAtomicResult::getCaseId, caseId));

        this.setIndexResultList(modelIndices, modelIndexResults, modelAtomicIndices, modelAtomicResults);

        return this;
    }
}
