package com.supervision.thread;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.supervision.police.domain.ModelRecordType;
import com.supervision.police.domain.NoteRecordSplit;
import com.supervision.police.service.NoteRecordSplitService;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.util.StopWatch;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.stream.Collectors;

/**
 * 笔录分类线程
 */
@Slf4j
public class RecordSplitTypeThread implements Callable<String> {


    private final List<ModelRecordType> allTypeList;

    private final NoteRecordSplit noteRecordSplit;

    private final OllamaChatClient chatClient;

    private final NoteRecordSplitService noteRecordSplitService;



    public RecordSplitTypeThread(List<ModelRecordType> allTypeList, NoteRecordSplit noteRecordSplit, OllamaChatClient chatClient,
                                 NoteRecordSplitService noteRecordSplitService) {
        this.allTypeList = allTypeList;
        this.chatClient = chatClient;
        this.noteRecordSplitService = noteRecordSplitService;
        this.noteRecordSplit = noteRecordSplit;
    }

    private static final String TYPE_TEMPLATE = """
             分类任务: 对话笔记录文本分类。目标: 将给定的对话笔记录分配到预定义的类别中，这些类别包括但不限于：{allTypes}。"
             说明: 提供一段对话笔记录文本，分类器应该识别出对话的主题，并将其归类到上述类别中的一个。
             示例输入: 文本:
             办案警官问：你为了骗取更多的钱都做了哪些准备？。裴金禄回答：我刚开始我就是自己想了一些关于骗钱的点子，后面为了更不容易让别人识破我为了更佳逼真，我就从网上随便搜了一家租赁公司，我就搜到了兰州胜利机械租赁有限公司，我又想到了我管理的中铁北京局和中铁电气化局施工公司。我先是通过百度搜索了“办证”之后就在网页上面弹出了一个页面上面有一个QQ号，我就加上了。加上之后我就将我的要求给他说了，要求他给我刻两个假的公章，一个是兰州胜利机械租赁有限公司合同专用章，另一个是中铁北京局集团有限公司合同专用章。我还要求他给我伪造了一张兰州胜利机械租赁有限公司的营业执照
             预期输出: { type: '诈骗准备' }
             任务要求:
             1. 分类器应当准确地识别对话的主题。
             2. 如果一段对话笔记录包含多个主题，请选择最相关的类别。
             3. 必须考虑上下文语境和专业术语来确定正确的分类。
             对话内容为：{question} {answer}
            """;

    private static final String NEW_TEMPLATE = """
            分类任务: 将对话笔录文本进行分类。
            目标: 将给定的对话分配到预定义的类别中
            预定义类别为：{typeContext}。
            说明: 提供一段对话笔记录文本，分类器应该识别出对话的主题，并将其归类到上述类别中的一个或多个。
            ---
            示例输入:
            办案警官问：你和上述的这些受害人签订协议之后是否实际履行合同？
            行为人XXX回答：我和他们签订合同就是为了骗他们相信我，我就是伪造的一些假合同，等我把他们的钱骗到手之后我就不会履行合同。
            预期输出: {"result":[{"type":"合同和协议","explain":"行为人XXX提到签订合同"},{"type":"虚假信息和伪造","explain":"行为人XXX提到合同是假合同"}]}
            ---
            任务要求:
            1. 分类器应当准确地识别对话的主题,分类来自于预定义的类别。
            2. 分类器应该实事求是按照对话进行分类,不要有过多的推测。
            2. 如果一段对话笔记录包含多个主题，请选择最相关的类别,最多可选择三个分类。
            3. 如果不涉及任何分类则回复{"result":[]}
            ---
            以下为问答对内容:
            {question}
            {answer}
            ---
            返回格式为json,字段名要严格一致:{"result":[{"type":"分类1","explain":"分类原因"},{"type":"分类2","explain":"分类原因"}]}
            """;
    private static final String TYPE_CONTEXT_TEMPLATE = "{分类type:{type},区别点(分类释义):{typeExt}}";

    @Override
    public String call() throws Exception {
        String type;
        try {

            StopWatch stopWatch = new StopWatch();
            // 首先拼接分类模板
            List<String> typeContextList = new ArrayList<>();
            for (ModelRecordType modelRecordType : allTypeList) {
                String format = StrUtil.format(TYPE_CONTEXT_TEMPLATE, Map.of("type", modelRecordType.getRecordType(), "typeExt", modelRecordType.getRecordTypeExt()));
                typeContextList.add(format);
            }
            // 开始对笔录进行分类
            Map<String, String> paramMap = new HashMap<>();
            paramMap.put("typeContext", CollUtil.join(typeContextList, ";"));
            paramMap.put("question", noteRecordSplit.getQuestion());
            paramMap.put("answer", noteRecordSplit.getAnswer());
            Prompt prompt = new Prompt(new UserMessage(StrUtil.format(NEW_TEMPLATE, paramMap)));
            stopWatch.start();
            log.info("开始分析:");
            ChatResponse call = chatClient.call(prompt);
            stopWatch.stop();
            log.info("耗时:{}", stopWatch.getTotalTimeSeconds());
            String content = call.getResult().getOutput().getContent();
            log.info("问：{}, 答：{}", noteRecordSplit.getQuestion(), noteRecordSplit.getAnswer());
            log.info("分析的结果是:{}", content);
            TypeResultDTO result = JSONUtil.toBean(content, TypeResultDTO.class);
            List<TypeNodeDTO> typeList = result.getResult();

            if (CollUtil.isNotEmpty(typeList)) {
                // 将type进行拼接,并以分号进行分割
                type = CollUtil.join(typeList.stream().map(TypeNodeDTO::getType).collect(Collectors.toSet()), ";");
            } else {
                // 如果没有提取到,就是无
                type = "无";
            }

        } catch (Exception e) {
            log.error("分类任务执行失败:{}", e.getMessage(), e);
            type = "无";
        }
        noteRecordSplitService.lambdaUpdate().set(NoteRecordSplit::getRecordType, type).eq(NoteRecordSplit::getId, noteRecordSplit.getId()).update();

        return  noteRecordSplit.getId();
    }


    @Data
    public static class TypeNodeDTO {
        private String type;
        private String explain;
    }

    @Data
    public static class TypeResultDTO {
        private List<TypeNodeDTO> result;
    }
}
